@@ -20,9 +20,9 @@
 #'   and deployment. The new data must be identical in structure to the data
 #'   that the recipe was prepared with.
 #'
-#' @param d A dataframe or tibble containing data to impute.
-#' @param ... Optional. Unquoted variable names to not be prepped. These will be
-#'   returned unaltered. Typically ID and outcome columns would go here.
+#' @param d A data frame
+#' @param ... Optional. Columns to be ignored in preparation and model training,
+#'   e.g. ID columns. Unquoted; any number of columns can be included here.
 #' @param outcome Optional. Unquoted column name that indicates the target
 #'   variable. If provided, argument must be named. If this target is 0/1, it
 #'   will be coerced to Y/N if factor_outcome is TRUE; other manipulation steps
@@ -33,12 +33,14 @@
 #'   prepared. If training data is big, pull the recipe from the "recipe"
 #'   attribute of the prepped training data frame and pass that to this
 #'   argument. If present, all following arguments will be ignored.
-#' @param remove_near_zero_variance Logical. If TRUE (default), columns with
-#'   near-zero variance will be removed. These columns are either a single
-#'   value, or meet both of the following criteria: 1. they have very few unique
-#'   values relative to the number of samples and 2. the ratio of the frequency
-#'   of the most common value to the frequency of the second most common value
-#'   is large.
+#' @param remove_near_zero_variance Logical or numeric. If TRUE (default),
+#'   columns with near-zero variance will be removed. These columns are either a
+#'   single value, or the most common value is much more frequent than the
+#'   second most common value.
+#'   Example: In a column with 120 "Male" and 2 "Female", the frequency ratio is
+#'   0.0167. It would be excluded by default or if
+#'   `remove_near_zero_variance` > 0.0166. Larger values will remove more columns
+#'   and this value must lie between 0 and 1.
 #' @param convert_dates Logical or character. If TRUE (default), date columns
 #'   are identifed and used to generate day-of-week, month, and year columns,
 #'   and the original date columns are removed. If FALSE, date columns are
@@ -65,8 +67,9 @@
 #'   added to all nominal columns. This is protective in deployment: new levels
 #'   found in deployment will become "other" and missingness in deployment can
 #'   become "missing" if the nominal imputation method is "new_category". If
-#'   FALSE, these levels may be added to some columns depending on details of
-#'   imputation and collapse_rare_factors.
+#'   FALSE, these "other" will be added to all nominal variables if
+#'   \code{collapse_rare_factors} is used, and "missingness" may be added
+#'   depending on details of imputation.
 #' @param factor_outcome Logical. If TRUE (default) and if all entries in
 #'   outcome are 0 or 1 they will be converted to factor with levels N and Y for
 #'   classification. Note that which level is the positive class is set in
@@ -104,7 +107,8 @@
 #'           impute = list(numeric_method = "bagimpute",
 #'                         nominal_method = "bagimpute"),
 #'           collapse_rare_factors = FALSE, convert_dates = "year",
-#'           center = TRUE, scale = TRUE, make_dummies = FALSE)
+#'           center = TRUE, scale = TRUE, make_dummies = FALSE,
+#'           remove_near_zero_variance = .02)
 prep_data <- function(d,
                       ...,
                       outcome,
@@ -124,6 +128,8 @@ prep_data <- function(d,
   }
   # Capture pre-modification missingness
   d_missing <- missingness(d, return_df = FALSE)
+  # Capture original data structure
+  d_ods <- d[0, ]
   # Capture factor levels
   d_levels <- get_factor_levels(d)
 
@@ -137,7 +143,7 @@ prep_data <- function(d,
   if (length(ignored)) {
     present <- ignored %in% names(d)
     if (any(!present))
-      stop(paste(ignored[!present], collapse = ", "), " not found in d.")
+      stop(list_variables(ignored[!present]), " not found in d.")
     if (length(ignored) >= ncol(d))
       stop("You only have ignored columns. Try again.")
     # Separate data into ignored and not
@@ -148,7 +154,7 @@ prep_data <- function(d,
       dplyr::filter(percent_missing > 0)
     if (!purrr::is_empty(m$variable))
       warning("These ignored variables have missingness: ",
-              paste(m$variable, collapse = ", "))
+              list_variables(m$variable))
   }
 
   # Check global options for factor handling
@@ -170,7 +176,7 @@ prep_data <- function(d,
                                    attr(recipe, "ignored_columns")))
     if (length(newvars)) {
       warning("These variables were not observed in training ",
-              "and will be ignored: ", paste(newvars, collapse = ", "))
+              "and will be ignored: ", list_variables(newvars))
       ignored <- c(ignored, newvars)
       d_ignore <- dplyr::bind_cols(d_ignore, dplyr::select(d, !!newvars))
     }
@@ -178,13 +184,13 @@ prep_data <- function(d,
     missing_vars <- setdiff(recipe$var_info$variable[recipe$var_info$role == "predictor"], names(d))
     if (length(missing_vars))
       stop("These variables were present in training but are missing or ignored here: ",
-           paste(missing_vars, collapse = ", "))
+           list_variables(missing_vars))
 
     # If imputing, look for variables with missingness now that didn't have any in training
     newly_missing <- find_new_missingness(d, recipe)
     if (length(newly_missing))
       warning("The following variable(s) have missingness that was not present when recipe was trained: ",
-              paste(newly_missing, collapse = ", "))
+              list_variables(newly_missing))
 
     # Outcome gets added as all NAs; set a flag to remove it at end if not in provided DF
     outcome_var <- recipe$var_info$variable[recipe$var_info$role == "outcome"]
@@ -204,7 +210,7 @@ prep_data <- function(d,
     }
 
     # Initialize a new recipe
-    mes <- "Training new data prep recipe"
+    mes <- "Training new data prep recipe...\n"
     ## Start by making all variables predictors...
     ## Only pass head of d here because it's carried around with the recipe
     ## and we don't want to pass around big datasets
@@ -248,9 +254,18 @@ prep_data <- function(d,
     # recipe <- recipe %>% step_hcai_mostly_missing_to_factor()  # nolint
 
     # Remove columns with near zero variance ----------------------------------
+    nzv_opts <- list(freq_cut = 49, unique_cut = 10)
+    if (!is.logical(remove_near_zero_variance)) {
+      if (!is.numeric(remove_near_zero_variance))
+        stop("remove_near_zero_variance must be logical or numeric for step_nzv")
+      if (remove_near_zero_variance < 0 | remove_near_zero_variance > 1)
+        stop("remove_near_zero_variance must be numeric between 0 and 1")
+      nzv_opts$freq_cut <- remove_near_zero_variance ^ -1
+      remove_near_zero_variance <- TRUE
+    }
     if (remove_near_zero_variance) {
       recipe <- recipe %>%
-        recipes::step_nzv(all_predictors())
+        recipes::step_nzv(all_predictors(), options = nzv_opts)
     }
 
     # Check if there are any nominal predictors that won't be removed; stop if not.
@@ -261,11 +276,10 @@ prep_data <- function(d,
     if (length(nom_preds) && all(nom_preds %in% removing))
       stop("All your categorical columns will be removed because they have ",
            "near-zero variance, which will break prep_data. ",
-           "Please let us know that you've encountered this error ",
-           "here - https://github.com/HealthCatalyst/healthcareai-r/issues/new - ",
-           "and we will work on a fix. In the meantime, either remove these ",
-           "NZV columns before prep_data or set remove_near_zero_variance = FALSE.\n  ",
-           paste(removing, collapse = ", "))
+           "Be less aggressive in removing near-zero variance columns by ",
+           "using a larger value of remove_near_zero_variance or setting it ",
+           "to FALSE.\n  ",
+           list_variables(removing))
 
     # Convert date columns to useful features and remove original. ------------
     if (!is.logical(convert_dates)) {
@@ -305,8 +319,8 @@ prep_data <- function(d,
       extras  <- names(impute)[!(names(impute) %in% names(ip))]
       if (length(extras > 0)) {
         warning("You have extra imputation parameters that won't be used: ",
-                paste(extras, collapse = ", "),
-                ". Available params are: ", paste(names(ip), collapse = ", "))
+                list_variables(extras),
+                ". Available params are: ", list_variables(names(ip)))
       }
 
       # Impute takes defaults or user specified inputs. Error handling inside.
@@ -392,16 +406,20 @@ prep_data <- function(d,
       length(nzv_removed <- recipe$steps[[which(steps == "step_nzv")]]$removals))
     message("Removing the following ", length(nzv_removed), " near-zero variance column(s). ",
             "If you don't want to remove them, call prep_data with ",
-            "remove_near_zero_variance = FALSE.\n  ",
-            paste(nzv_removed, collapse = ", "))
+            "remove_near_zero_variance as a smaller numeric or FALSE.\n  ",
+            list_variables(nzv_removed))
 
   # Remove outcome if recipe was provided but outcome not present
   if (remove_outcome && outcome_var %in% names(d))
     d <- select_not(d, outcome_var)
+  # And remove outcome from original_data_str if it's present
+  if (rlang::quo_name(outcome) %in% names(d_ods))
+    d_ods <- select_not(d_ods, outcome)
   # Add ignore columns back in and attach as attribute to recipe
   d <- dplyr::bind_cols(d_ignore, d)
   attr(recipe, "ignored_columns") <- unname(ignored)
   attr(d, "recipe") <- recipe
+  attr(d, "original_data_str") <- d_ods
   d <- tibble::as_tibble(d)
   class(d) <- c("prepped_df", class(d))
 
